@@ -12,8 +12,10 @@ import speckenv
 from fabric import Connection, task
 from invoke import Collection  # noqa, re-export
 
+from fh_fablib.extract_js_gettext_strings import generate_strings
 
-__version__ = "1.0.20210202"
+
+__version__ = "1.0.20211029"
 
 
 # I don't care, in this context.
@@ -27,12 +29,17 @@ def ansi(code):
 # underline = ansi("4")
 red = ansi("31")
 green = ansi("32")
+purple = ansi("35")
 
 
 def progress(msg):
     print(green(msg))
 
 
+def info(msg):
+    print(purple(msg))
+
+
 def terminate(msg):
     print(red(msg), file=sys.stderr)
     sys.exit(1)
@@ -41,6 +48,24 @@ def terminate(msg):
 def require(version):
     if __version__ < version:
         terminate(f"fh_fablib version {version} required (you have {__version__})")
+    if __version__ > version:
+        path = Path.cwd() / "fabfile.py"
+        if path.is_file() and (old := path.read_text()):
+            new = "".join(
+                f'fl.require("{__version__}")\n'
+                if line.startswith("fl.require")
+                else line
+                for line in old.splitlines(keepends=True)
+            )
+            if new != old:
+                path.write_text(new)
+                info(
+                    "The fabfile has been updated automatically with your"
+                    " local fh_fablib version."
+                )
+                return
+
+        info(f"fh_fablib version is {__version__}, project requires only {version}.")
 
 
 def run(c, *a, **kw):
@@ -55,6 +80,7 @@ class Config:
     app = "app"
     environment = "default"
     environments = {}
+    force = False
 
     def update(self, **kwargs):
         for key, value in kwargs.items():
@@ -159,7 +185,10 @@ def dev(ctx, host="127.0.0.1", port=8000):
         ctx,
         [
             f"venv/bin/python manage.py runserver 0.0.0.0:{port}",
-            f'HOST="{host}" npx webpack-dev-server --host 0.0.0.0 --port 4000 --hot',
+            (
+                f'HOST="{host}" yarn run webpack-dev-server'
+                " --host 0.0.0.0 --port 4000 --hot"
+            ),
         ],
     )
 
@@ -181,6 +210,14 @@ def pull_db(ctx):
     reset_pw(ctx)
 
 
+@task(auto_shortflags=False)
+def pull_media(ctx, folder="media"):
+    """Rsync a folder from the remote to the local environment"""
+    flags = "-pthrz --stats"
+    folder = folder.strip("/")
+    run(ctx, f"rsync {flags} {config.host}:{config.domain}/{folder}/ {folder}/")
+
+
 @task
 def reset_pw(ctx):
     """Set all user passwords to "password" """
@@ -211,19 +248,27 @@ def _srv_env(conn, path):
     return lambda *a, **kw: speckenv.env(*a, **kw, mapping=mapping)
 
 
-@task
-def mm(ctx):
+@task(
+    auto_shortflags=False,
+    help={"language": "Generate catalogs for a specific language"},
+)
+def mm(ctx, language=None):
     """Update the translation catalogs"""
+
+    with io.open("conf/strings.js", "w", encoding="utf-8") as f:
+        f.write("".join(f"{str}\n" for str in generate_strings()))
+
+    language = f"-l {language}" if language else "-a"
     run(
         ctx,
-        "venv/bin/python manage.py makemessages -a --add-location file"
-        " -i venv -i htmlcov -i node_modules -i lib",
+        f"venv/bin/python manage.py makemessages {language} --add-location file"
+        " -i venv -i htmlcov -i node_modules -i lib -i build -i dist",
         replace_env=False,
     )
     run(
         ctx,
-        "venv/bin/python manage.py makemessages -a --add-location file"
-        " -i venv -i htmlcov -i node_modules -i lib"
+        f"venv/bin/python manage.py makemessages {language} --add-location file"
+        " -i venv -i htmlcov -i node_modules -i lib -i build -i dist"
         " -d djangojs",
         replace_env=False,
     )
@@ -235,7 +280,7 @@ def cm(ctx):
     run(
         ctx,
         "venv/bin/python manage.py compilemessages"
-        " -i venv -i htmlcov -i node_modules -i lib",
+        " -i venv -i htmlcov -i node_modules -i lib -i build -i dist",
         replace_env=False,
     )
 
@@ -267,7 +312,7 @@ def freeze(ctx):
         ctx,
         '(printf "# AUTOGENERATED, DO NOT EDIT\n\n";' "venv/bin/python -m pip freeze -l"
         # Until Ubuntu gets its act together:
-        ' | grep -vE "(^pkg-resources)"' ") > requirements.txt",
+        ' | grep -vE "(^pkg.resources)"' ") > requirements.txt",
     )
 
 
@@ -281,7 +326,7 @@ def update(ctx):
     run(ctx, "venv/bin/python -m pip install -r requirements.txt")
     run(ctx, 'find . -name "*.pyc" -delete')
     run(ctx, "yarn")
-    run(ctx, "venv/bin/python manage.py migrate")
+    run(ctx, "venv/bin/python manage.py migrate", warn=True)
 
 
 def _local_dotenv_if_not_exists():
@@ -302,6 +347,7 @@ SECRET_KEY={secret_key}
 ALLOWED_HOSTS=["*"]
 
 SENTRY_DSN=
+SENTRY_ENVIRONMENT=
 """
         )
 
@@ -311,10 +357,14 @@ def _local_dbname():
     return _dbname_from_dsn(_local_env()("DATABASE_URL"))
 
 
-@task
-def local(ctx):
+@task(
+    auto_shortflags=False,
+    help={"clobber": "Clobber pre-existing node_modules and venv folders"},
+)
+def local(ctx, clobber=False):
     """Local environment setup"""
-    run(ctx, "rm -rf node_modules venv")
+    if clobber:
+        run(ctx, "rm -rf node_modules venv")
     dbname = _local_dbname()
     run(ctx, f"createdb {dbname}", warn=True)
     update(ctx)
@@ -429,11 +479,13 @@ DEBUG=False
 DATABASE_URL=postgres://{dbname}:{password}@localhost:5432/{dbname}
 CACHE_URL=hiredis://localhost:6379/1/?key_prefix={dbname}
 SECRET_KEY={secret_key}
-ALLOWED_HOSTS=[".{config.domain}", ".{conn.host}"]
+ALLOWED_HOSTS=[".{config.domain}", ".{conn.host}", ".feinheit.dev"]
 
 GOOGLE_CLIENT_ID=
 GOOGLE_CLIENT_SECRET=
+
 SENTRY_DSN=
+SENTRY_ENVIRONMENT=
 
 # LIVE=True
 # CANONICAL_DOMAIN={config.domain}
@@ -619,7 +671,7 @@ def fetch(ctx):
 
 
 def _check_flake8(ctx):
-    run(ctx, "pipx run --spec 'flake8>=3.8.3' flake8 .")
+    run(ctx, "pipx run flake8 .")
 
 
 def _check_django(ctx):
@@ -629,13 +681,13 @@ def _check_django(ctx):
 def _check_prettier(ctx):
     run(
         ctx,
-        f"npx prettier --list-different --no-semi"
+        f"yarn run prettier --list-different --no-semi"
         f' "*.js" "{config.app}/static/**/*.js" "{config.app}/static/**/*.scss"',
     )
 
 
 def _check_eslint(ctx):
-    run(ctx, f'npx eslint "*.js" {config.app}/static')
+    run(ctx, f'yarn run eslint "*.js" {config.app}/static')
 
 
 def _check_large_files(ctx, *, limit=500000):
@@ -678,40 +730,57 @@ def check(ctx):
 def _fmt_prettier(ctx):
     run(
         ctx,
-        f"npx prettier --write --no-semi"
+        f"yarn run prettier --write --no-semi"
         f' "*.js" "{config.app}/static/**/*.js" "{config.app}/static/**/*.scss"',
+        warn=True,
     )
 
 
 def _fmt_tox_style(ctx):
-    run(ctx, "tox -e style")
+    run(ctx, "tox -e style", warn=True)
+
+
+def _fmt_pyupgrade(ctx):
+    run(ctx, "pipx run pyupgrade --py36-plus $(git ls-files '*.py')", warn=True)
 
 
 def _fmt_black(ctx):
-    run(ctx, "pipx run --spec 'black>=20.8b1' black .")
+    run(ctx, "pipx run black .", warn=True)
 
 
 def _fmt_isort(ctx):
     run(
         ctx,
-        "pipx run --spec 'isort>=5.4' isort"
+        "pipx run isort"
         " --virtual-env venv --profile=black --lines-after-imports=2 --combine-as"
         " .",
+        warn=True,
     )
 
 
+def _fmt_djlint(ctx):
+    run(ctx, f"pipx run djlint --reformat --quiet {config.app}", warn=True)
+
+
 @task
 def fmt(ctx):
     """Format the code"""
+    _fmt_pyupgrade(ctx)
     _fmt_black(ctx)
     _fmt_isort(ctx)
+    _fmt_djlint(ctx)
     _fmt_prettier(ctx)
 
 
 def _deploy_django(conn):
-    run(conn, f"git checkout {config.branch}")
     run(conn, "git fetch origin")
-    run(conn, f"git merge --ff-only origin/{config.branch}")
+    run(conn, f"git checkout {config.branch}")
+
+    result = run(conn, "git status --porcelain").stdout.strip()
+    if result:
+        terminate("Terminating because of uncommitted changes on server")
+
+    run(conn, f"git reset --hard origin/{config.branch}")
     run(conn, 'find . -name "*.pyc" -delete')
     _pip_up(conn)
     run(conn, "venv/bin/python -m pip install -r requirements.txt")
@@ -729,15 +798,19 @@ def _rsync_static(ctx, *, delete=False):
     run(ctx, f"rsync {flags}{delete} static/ {config.host}:{config.domain}/static/")
 
 
-@task(auto_shortflags=False, help={"fast": "Skip the Webpack build"})
-def deploy(ctx, fast=False):
+@task(
+    auto_shortflags=False,
+    help={"fast": "Skip the Webpack build", "force": "Force the git push"},
+)
+def deploy(ctx, fast=False, force=False):
     """Deploy once 🔥"""
     _check_branch(ctx)
     _check_no_uncommitted_changes(ctx)
     check(ctx)
-    run(ctx, f"git push origin {config.branch}")
+    force = "+" if (force or config.force) else ""
+    run(ctx, f"git push origin {force}{config.branch}")
     if not fast:
-        run(ctx, "NODE_ENV=production npx webpack -p --bail")
+        run(ctx, "NODE_ENV=production yarn run webpack -p --bail")
 
     with Connection(config.host) as conn, conn.cd(config.domain):
         _deploy_django(conn)
@@ -759,6 +832,7 @@ GENERAL = {
     freeze,
     update,
     pull_db,
+    pull_media,
     reset_pw,
     local,
     bitbucket,
